;;;; message-digest-test.scm

;; Issues
;;
;; - Needs many more tests, especially the entire input-port & procedure source stuff.

(use test)
(use message-digest message-digest-port)
(use files lolevel srfi-4)

;;

(define simple-src "ab cd")
(define simple-res "6162206364")

(define digest-length 5)
(define context-size 10)

(test-group "Message Digest Aux"
	(test simple-res (byte-string->hexadecimal simple-src))
	(test (substring simple-res 4 6) (byte-string->hexadecimal simple-src 2 3))
)

(test-group "Message Digest Make Primitive"

  (define (init ctx) (void))
  (define (update ctx bytes count) (void))
  (define (final ctx result) (void))

  (let ((mdp (make-message-digest-primitive context-size digest-length init update final 'foo)))
    (test-assert (message-digest-primitive? mdp))
    (test context-size (message-digest-primitive-context-info mdp))
    (test digest-length (message-digest-primitive-digest-length mdp))
    (test init (message-digest-primitive-init mdp))
    (test update (message-digest-primitive-update mdp))
    (test final (message-digest-primitive-final mdp))
    (test 'foo (message-digest-primitive-name mdp)) )
)

;These also test the update-string proc
(test-group "Message Digest Proper Phase Arguments (Def Alloc)"

  (define (init ctx)
    #;(printf "  Init Ctx: ~S~%" ctx)
    (assert (pointer? ctx)) )

  (define (update ctx bytes count)
    #;(printf "Update Ctx: ~S Bytes: ~S Count: ~S~%" ctx bytes count)
    (assert (pointer? ctx))
    (assert (string? bytes))
    (assert (<= count (string-length bytes)))
    (assert (>= context-size count))  ; So no mem overflow
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    #;(printf " Final Ctx: ~S Result: ~S~%" ctx result)
    (assert (pointer? ctx))
    (assert (string? result))
    (assert (= digest-length (string-length result)))  ; So no mem overflow
    (move-memory! ctx result digest-length) )

  (let ((mdp (make-message-digest-primitive context-size digest-length init update final)))
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest? md))
      (test-assert (message-digest-update-string md simple-src))
      (test simple-res (finalize-message-digest md)) ) )
)

(test-group "Message Digest Proper Phase Arguments (Own Alloc)"

  (define (make-context) (make-blob context-size))

  (define (init ctx)
    #;(printf "  Init Ctx: ~S~%" ctx)
    (assert (blob? ctx)) )

  (define (update ctx bytes count)
    #;(printf "Update Ctx: ~S Bytes: ~S Count: ~S~%" ctx bytes count)
    (assert (blob? ctx))
    (assert (string? bytes))
    (assert (<= count (string-length bytes)))
    (assert (>= context-size count))  ; So no mem overflow
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    #;(printf " Final Ctx: ~S Result: ~S~%" ctx result)
    (assert (blob? ctx))
    (assert (string? result))
    (assert (= digest-length (string-length result)))  ; So no mem overflow
    (move-memory! ctx result digest-length) )

  (let ((mdp (make-message-digest-primitive make-context digest-length init update final)))
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest? md))
      (test-assert (message-digest-update-string md simple-src))
      (test simple-res (finalize-message-digest md)) ) )
)

(let ()

  (define (make-context)
    (string->blob (make-string context-size #\nul)) )

  (define (init ctx) (void))

  (define (update ctx bytes count)
    (assert (>= context-size count))  ; So no mem overflow
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    (assert (= digest-length (string-length result)))  ; So no mem overflow
    (move-memory! ctx result digest-length) )

  (define mdp (make-message-digest-primitive make-context digest-length init update final))
    
  (test-group "Message Digest u8vector Source"
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest-update-u8vector md (u8vector 1 2 3 4 5)))
      (test "0102030405" (finalize-message-digest md)) )
  )

  (test-group "Message Digest u8 Source"
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest-update-u8 md #xA2))
      (test "A200000000" (finalize-message-digest md)) )
  )
  
  (test-group "Message Digest u16-le Source"
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest-update-u16-le md #xA2B2))
      (test "B2A2000000" (finalize-message-digest md)) )
  )
  
  (test-group "Message Digest u32-be Source"
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest-update-u32-be md #xA2B2C2D2))
      (test "A2B2C2D200" (finalize-message-digest md)) )
  )
  
  (test-group "Message Digest Procedure Source"
    (define just-once
      (let ((x #t))
        (lambda ()
          (let ((res (and x simple-src)))
            (set! x #f)
            res ) ) ) )
    (let ((md (initialize-message-digest mdp)))
      (test-assert (message-digest-update-procedure md just-once))
      (test simple-res (finalize-message-digest md)) )
  )

  (test-group "Message Digest Port"
    (let ((port (open-output-digest mdp)))
      (test-assert (output-port? port))
      (display simple-src port) ;cannot be readable!
      (test simple-res (get-output-digest-hexstring port))
      (test-assert (port-closed? port)) )
  )
)

(let ()

  (define (init ctx) (void))

  (define (update ctx bytes count)
    (assert (>= context-size count))  ; So no mem overflow
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    (assert (= digest-length (string-length result)))  ; So no mem overflow
    (move-memory! ctx result digest-length) )

  (test-group "Message Digest Primitive Apply (DEPRECATED)"
    (let ((mdp (make-message-digest-primitive context-size digest-length init update final)))
      (let ((res (message-digest-primitive-apply mdp simple-src)))
        (test-assert (string? res))
        (test simple-res (byte-string->hexadecimal res)) ) )
  )
  
  (test-group "Message Digest Make (DEPRECATED)"
    (test simple-src (make-binary-message-digest simple-src make-context digest-length init update final))
    (test simple-res (make-message-digest simple-src make-context digest-length init update final))
  )
)
